@@ -48,6 +48,21 @@ getdns_dict  dnssec_ok_checking_disabled_spc = {
 };
 getdns_dict *dnssec_ok_checking_disabled = &dnssec_ok_checking_disabled_spc;
 
+getdns_dict  dnssec_ok_checking_disabled_roadblock_avoidance_spc = {
+	{ RBTREE_NULL, 0, (int (*)(const void *, const void *)) strcmp },
+	{ 0 }
+};
+getdns_dict *dnssec_ok_checking_disabled_roadblock_avoidance
+    = &dnssec_ok_checking_disabled_roadblock_avoidance_spc;
+
+getdns_dict  dnssec_ok_checking_disabled_avoid_roadblocks_spc = {
+	{ RBTREE_NULL, 0, (int (*)(const void *, const void *)) strcmp },
+	{ 0 }
+};
+getdns_dict *dnssec_ok_checking_disabled_avoid_roadblocks
+    = &dnssec_ok_checking_disabled_avoid_roadblocks_spc;
+
+
 static int
 is_extension_set(getdns_dict *extensions, const char *extension)
 {
@@ -56,7 +71,9 @@ is_extension_set(getdns_dict *extensions, const char *extension)
 
 	if (! extensions)
 		return 0;
-	else if (extensions == dnssec_ok_checking_disabled)
+	else if (extensions == dnssec_ok_checking_disabled
+	    || extensions == dnssec_ok_checking_disabled_roadblock_avoidance
+	    || extensions == dnssec_ok_checking_disabled_avoid_roadblocks)
 		return 0;
 
 	r = getdns_dict_get_int(extensions, extension, &value);
@@ -113,6 +130,13 @@ network_req_init(getdns_network_req *net_req, getdns_dns_req *owner,
 	                              ? edns_maximum_udp_payload_size : 1432;
 	net_req->write_queue_tail = NULL;
 	net_req->response_len = 0;
+        net_req->base_query_option_sz = opt_options_size;
+
+	/* Some fields to record info for return_call_debugging */
+	net_req->debug_start_time = 0;
+	net_req->debug_end_time = 0;
+	net_req->debug_tls_auth_status = 0;
+	net_req->debug_udp = 0;
 
 	net_req->wire_data_sz = wire_data_sz;
 	if (max_query_sz) {
@@ -184,6 +208,75 @@ network_req_init(getdns_network_req *net_req, getdns_dns_req *owner,
 	return r;
 }
 
+/* req->opt + 9 is the length; req->opt + 11 is the start of the
+   options. 
+
+   clear_upstream_options() goes back to the per-query options.
+ */
+
+void
+_getdns_network_req_clear_upstream_options(getdns_network_req * req)
+{
+  size_t pktlen;
+  if (req->opt) {
+	  gldns_write_uint16(req->opt + 9, req->base_query_option_sz);
+	  req->response = req->opt + 11 + req->base_query_option_sz;
+	  pktlen = req->response - req->query;
+	  gldns_write_uint16(req->query - 2, pktlen);
+  }
+}
+
+/* add_upstream_option appends an option that is derived at send time.
+    (you can send data as NULL and it will fill with all zeros) */
+getdns_return_t
+_getdns_network_req_add_upstream_option(getdns_network_req * req, uint16_t code, uint16_t sz, const void* data)
+{
+  uint16_t oldlen;
+  uint32_t newlen;
+  uint32_t pktlen;
+  size_t cur_upstream_option_sz;
+
+  /* if no options are set, we can't add upstream options */
+  if (!req->opt)
+	  return GETDNS_RETURN_GENERIC_ERROR;
+  
+  /* if TCP, no overflow allowed for length field
+     https://tools.ietf.org/html/rfc1035#section-4.2.2 */
+  pktlen = req->response - req->query;
+  pktlen += 4 + sz;
+  if (pktlen > UINT16_MAX)
+    return GETDNS_RETURN_GENERIC_ERROR;
+  
+  /* no overflow allowed for OPT size either (maybe this is overkill
+     given the above check?) */
+  oldlen = gldns_read_uint16(req->opt + 9);  
+  newlen = oldlen + 4 + sz;
+  if (newlen > UINT16_MAX)
+    return GETDNS_RETURN_GENERIC_ERROR;
+
+  /* avoid overflowing MAXIMUM_UPSTREAM_OPTION_SPACE */
+  cur_upstream_option_sz = (size_t)oldlen - req->base_query_option_sz;
+  if (cur_upstream_option_sz  + 4 + sz > MAXIMUM_UPSTREAM_OPTION_SPACE)
+    return GETDNS_RETURN_GENERIC_ERROR;
+
+  /* actually add the option: */
+  gldns_write_uint16(req->opt + 11 + oldlen, code);
+  gldns_write_uint16(req->opt + 11 + oldlen + 2, sz);
+  if (data != NULL)
+	  memcpy(req->opt + 11 + oldlen + 4, data, sz);
+  else
+	  memset(req->opt + 11 + oldlen + 4, 0, sz);
+  gldns_write_uint16(req->opt + 9, newlen);
+
+  /* the response should start right after the options end: */
+  req->response = req->opt + 11 + newlen;
+  
+  /* for TCP, adjust the size of the wire format itself: */
+  gldns_write_uint16(req->query - 2, pktlen);
+  
+  return GETDNS_RETURN_GOOD;
+}
+
 void
 _getdns_dns_req_free(getdns_dns_req * req)
 {
@@ -221,10 +314,24 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 	    =  is_extension_set(extensions, "dnssec_return_validation_chain");
 	int edns_cookies
 	    =  is_extension_set(extensions, "edns_cookies");
+#ifdef DNSSEC_ROADBLOCK_AVOIDANCE
+	int avoid_dnssec_roadblocks
+	    =  (extensions == dnssec_ok_checking_disabled_avoid_roadblocks);
+	int dnssec_roadblock_avoidance
+	    = is_extension_set(extensions, "dnssec_roadblock_avoidance")
+	    || (extensions == dnssec_ok_checking_disabled_roadblock_avoidance)
+	    || avoid_dnssec_roadblocks;
+#endif
 
 	int dnssec_extension_set = dnssec_return_status
 	    || dnssec_return_only_secure || dnssec_return_validation_chain
-	    || (extensions == dnssec_ok_checking_disabled);
+	    || (extensions == dnssec_ok_checking_disabled)
+	    || (extensions == dnssec_ok_checking_disabled_roadblock_avoidance)
+	    || (extensions == dnssec_ok_checking_disabled_avoid_roadblocks)
+#ifdef DNSSEC_ROADBLOCK_AVOIDANCE
+	    || dnssec_roadblock_avoidance
+#endif
+	    ;
 
 	uint32_t edns_do_bit;
 	int      edns_maximum_udp_payload_size;
@@ -259,7 +366,9 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 	size_t max_query_sz, max_response_sz, netreq_sz, dnsreq_base_sz;
 	uint8_t *region;
 	
-	if (extensions == dnssec_ok_checking_disabled)
+	if (extensions == dnssec_ok_checking_disabled ||
+	    extensions == dnssec_ok_checking_disabled_roadblock_avoidance ||
+	    extensions == dnssec_ok_checking_disabled_avoid_roadblocks)
 		extensions = NULL;
 
 	have_add_opt_parameters = getdns_dict_get_dict(extensions,
@@ -297,7 +406,8 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 
 	with_opt = edns_do_bit != 0 || edns_maximum_udp_payload_size != 512 ||
 	    edns_extended_rcode != 0 || edns_version != 0 || noptions ||
-	    edns_cookies;
+	    edns_cookies || context->edns_client_subnet_private ||
+	    context->tls_query_padding_blocksize > 1;
 
 	edns_maximum_udp_payload_size = with_opt &&
 	    ( edns_maximum_udp_payload_size == -1 ||
@@ -305,7 +415,12 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 	    ? edns_maximum_udp_payload_size : 512;
 
 	/* (x + 7) / 8 * 8 to align on 8 byte boundries */
+#ifdef DNSSEC_ROADBLOCK_AVOIDANCE
+	if (context->resolution_type == GETDNS_RESOLUTION_RECURSING
+	    && (!dnssec_roadblock_avoidance || avoid_dnssec_roadblocks)) 
+#else
 	if (context->resolution_type == GETDNS_RESOLUTION_RECURSING)
+#endif
 		max_query_sz = 0;
 	else {
 		for (i = 0; i < noptions; i++) {
@@ -323,12 +438,7 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 		max_query_sz = ( GLDNS_HEADER_SIZE
 		    + strlen(name) + 1 + 4 /* dname always smaller then strlen(name) + 1 */
 		    + 12 + opt_options_size /* space needed for OPT (if needed) */
-		    + ( !edns_cookies ? 0
-		      :  2 /* EDNS0 Option Code */
-		      +  2 /* Option length = 8 + 16 = 24 */
-		      +  8 /* client cookie */
-		      + 16 /* server cookie */
-		      )
+		    + MAXIMUM_UPSTREAM_OPTION_SPACE
 		    /* TODO: TSIG */
 		    + 7) / 8 * 8;
 	}
@@ -372,7 +482,15 @@ _getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
 	result->dnssec_return_only_secure      = dnssec_return_only_secure;
 	result->dnssec_return_validation_chain = dnssec_return_validation_chain;
 	result->edns_cookies                   = edns_cookies;
-
+#ifdef DNSSEC_ROADBLOCK_AVOIDANCE
+	result->dnssec_roadblock_avoidance     = dnssec_roadblock_avoidance;
+	result->avoid_dnssec_roadblocks        = avoid_dnssec_roadblocks;
+#endif
+	result->edns_client_subnet_private     = context->edns_client_subnet_private;
+	result->tls_query_padding_blocksize    = context->tls_query_padding_blocksize;
+	result->return_call_debugging
+		= is_extension_set(extensions, "return_call_debugging");
+	
 	/* will be set by caller */
 	result->user_pointer = NULL;
 	result->user_callback = NULL;
